@@ -3,7 +3,7 @@ from __future__ import absolute_import
 
 import json
 import keyword
-import requests
+from requests import RequestException as requests_RequestException, ConnectionError as requests_ConnectionError, exceptions as requests_exceptions, codes as requests_codes
 
 from .network import CFnetwork
 from .logging_helper import CFlogger
@@ -38,8 +38,14 @@ class CloudFlare():
 
             self.raw = config['raw']
             self.use_sessions = config['use_sessions']
+            self.global_request_timeout = config['global_request_timeout'] if 'global_request_timeout' in config else None
+            self.max_request_retries = config['max_request_retries'] if 'max_request_retries' in config else None
             self.profile = config['profile']
-            self.network = CFnetwork(use_sessions=self.use_sessions)
+            self.network = CFnetwork(
+                use_sessions=self.use_sessions,
+                global_request_timeout=self.global_request_timeout,
+                max_request_retries=self.max_request_retries
+            )
             self.user_agent = user_agent()
 
             self.logger = CFlogger(config['debug']).getLogger() if 'debug' in config and config['debug'] else None
@@ -194,21 +200,29 @@ class CloudFlare():
                     if parts[4]:
                         url += '/' + parts[4]
 
+            if files and data:
+                # Can't send data and form data - so move data into files and send as multipart/form-data
+                new_files = []
+                new_files += [(f, (files[f].name, files[f])) for f in files]
+                new_files += [(d, (None, data[d])) for d in data]
+                files = tuple(new_files)
+                data = None
+
             if self.logger:
                 msg = build_curl(method, url, headers, params, data, files)
                 self.logger.debug('Call: emulated curl command ...\n%s', msg)
 
             try:
                 response = self.network(method, url, headers, params, data, files)
-            except requests.RequestException as e:
+            except requests_RequestException as e:
                 if self.logger:
                     self.logger.debug('Call: requests exception! "%s"', e)
                 raise CloudFlareAPIError(0, e)
-            except requests.ConnectionError as e:
+            except requests_ConnectionError as e:
                 if self.logger:
                     self.logger.debug('Call: requests connection exception! "%s"', e)
                 raise CloudFlareAPIError(0, 'connection error')
-            except requests.exceptions.Timeout as e:
+            except requests_exceptions.Timeout as e:
                 if self.logger:
                     self.logger.debug('Call: requests timeout exception! "%s"', e)
                 raise CloudFlareAPIError(0, 'connection timeout')
@@ -298,7 +312,7 @@ class CloudFlare():
                 except ValueError:
                     if response_data == '':
                         # This should really be 'null' but it isn't. Even then, it's wrong!
-                        if response_code == requests.codes.ok:
+                        if response_code == requests_codes.ok:
                             # 200 ok
                             response_data = {'success': True,
                                              'result': None}
@@ -321,7 +335,7 @@ class CloudFlare():
                                 self.logger.debug('Response data not JSON: %r', response_data)
                             raise CloudFlareAPIError(0, 'JSON parse failed - report to Cloudflare.')
 
-                if response_code == requests.codes.ok:
+                if response_code == requests_codes.ok:
                     # 200 ok - so nothing needs to be done
                     pass
                 else:
@@ -330,7 +344,7 @@ class CloudFlare():
                     pass
             elif response_type == 'application/octet-stream' and isinstance(response_data, (int, float)):
                 # It's binary data
-                if response_code == requests.codes.ok:
+                if response_code == requests_codes.ok:
                     # 200 ok
                     response_data = {'success': True,
                                      'result': response_data}
@@ -346,7 +360,7 @@ class CloudFlare():
                 try:
                     response_data = json.loads(response_data)
                     if not isinstance(response_data, (dict)) or 'success' not in response_data:
-                        if response_code == requests.codes.ok:
+                        if response_code == requests_codes.ok:
                             # 200 ok
                             response_data = {'success': True,
                                              'result': response_data}
@@ -358,7 +372,7 @@ class CloudFlare():
                 except ValueError:
                     # So it wasn't JSON - moving on as if it's text!
                     # A single value is returned (vs an array or object)
-                    if response_code == requests.codes.ok:
+                    if response_code == requests_codes.ok:
                         # 200 ok
                         response_data = {'success': True, 'result': response_data}
                     else:
@@ -378,7 +392,7 @@ class CloudFlare():
                 except ValueError:
                     # So it wasn't JSON - moving on as if it's text!
                     # A single value is returned (vs an array or object)
-                    if response_code == requests.codes.ok:
+                    if response_code == requests_codes.ok:
                         # 200 ok
                         response_data = {'success': True, 'result': response_data}
                     else:
@@ -388,7 +402,7 @@ class CloudFlare():
                                          'result': response_data}
             elif response_type in ['text/javascript', 'application/javascript']:
                 # used by Cloudflare workers
-                if response_code == requests.codes.ok:
+                if response_code == requests_codes.ok:
                     # 200 ok
                     response_data = {'success': True,
                                      'result': str(response_data)}
@@ -399,7 +413,7 @@ class CloudFlare():
                                      'result': str(response_data)}
             elif response_type == 'text/html':
                 # used by media for preview
-                if response_code == requests.codes.ok:
+                if response_code == requests_codes.ok:
                     # 200 ok
                     response_data = {'success': True,
                                      'result': str(response_data)}
@@ -412,7 +426,7 @@ class CloudFlare():
             else:
                 # Assuming nothing - but continuing anyway
                 # A single value is returned (vs an array or object)
-                if response_code == requests.codes.ok:
+                if response_code == requests_codes.ok:
                     # 200 ok
                     response_data = {'success': True,
                                      'result': str(response_data)}
@@ -531,13 +545,6 @@ class CloudFlare():
             result = response_data
             return result
 
-        def api_from_web(self):
-            """ Cloudflare v4 API"""
-
-            # base url isn't enough; we need less
-            url = '/'.join(self.base_url.split('/')[0:3])
-            return self._read_from_web(url)
-
         def api_from_openapi(self, url):
             """ Cloudflare v4 API"""
 
@@ -908,7 +915,7 @@ class CloudFlare():
 
         return api_decode_from_openapi(self._base.api_from_openapi(url))
 
-    def __init__(self, email=None, key=None, token=None, certtoken=None, debug=False, raw=False, use_sessions=True, profile=None, base_url=None):
+    def __init__(self, email=None, key=None, token=None, certtoken=None, debug=False, raw=False, use_sessions=True, profile=None, base_url=None, global_request_timeout=5, max_request_retries=5):
         """ Cloudflare v4 API"""
 
         self._base = None
@@ -937,6 +944,10 @@ class CloudFlare():
             config['profile'] = profile
         if base_url is not None:
             config['base_url'] = base_url
+        if global_request_timeout is not None:
+            config['global_request_timeout'] = global_request_timeout
+        if max_request_retries is not None:
+            config['max_request_retries'] = max_request_retries
 
         # we do not need to handle item.call values - they pass straight thru
 
